package jp.ac.takushoku_u.cs;
import javax.swing.JFrame;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;

/**
* Graphpaintクラスはグラフを描画するためのクラスです。
* @version 1.0
* @author 笠島 裕史(Kasajima Hiroshi)
* @see jp.ac.takushoku_u.cs.ArrangePoint
* @see jp.ac.takushoku_u.cs.SSGraphPoint
*/
/*
* 最終更新日 2005年03月01日
*/
public class GraphPaint extends JFrame{
	
/**
* 隣接マトリクス
*/
	private BooleanMatrix matrix;
	
/**
* 可到達マトリクス
*/
	private BooleanMatrix reachMatrix;
	
/**
* 最大レベル(レベルは0から始まるため実際のレベル数は最大レベル+1)
*/
	private int maxLevel;
	
/**
* 最大横幅数
*/
	private int maxWide;
	
/**
* 描画要素数
*/
	private int elementNumber;
	
/**
* 描画レベル配列(配列数は要素数)
*/
	private int[] paintLevel;
	
/**
* 同一レベルでの描画位置
*/
	private int[] paintNumber;
	
/**
* 横幅配列(配列数はレベル数)
*/
	private int[] paintWide;
	
/**
* 各要素のx,y座標
*/
	private int[][] paintPoint;
	
/**
* グラフに描く要素のサイズ
*/
	private int elmentSize = 20;
	
/**
* 1要素のブロック
*/
	private int elementBlock = 50;
	
/**
* コンストラクタを抜けたかどうか
*/
	private boolean isCon = false;
	
/**
* フレームのx値
*/
	private int xp;
/**
* フレームのy値
*/
	private int yp;
	
/**
* 描画する縦軸の最大値
*/
	private double maxValue = Double.NaN;
	
/**
* 縦軸の目盛描画数
*/
	private int scale = 10;
	
/**
* バックカラー
*/
	private Color bkColor = Color.white;
	
/**
* 矢の色
*/
	private Color arColor = Color.red;
	
/**
* 縦軸線の色
*/
	private Color rowColor = Color.magenta;
	
/**
* 縦軸の目盛の色
*/
	private Color rowNumberColor = Color.darkGray;
	
/**
* 描画する要素の色
*/
	private Color elColor = Color.cyan;
	
/**
* 要素内の番号の色
*/
	private Color elNumberColor = Color.black;
	
	
/**
* 要素のない隣接マトリクスを設定し、描画を行います。
*/
	public GraphPaint(){
		super("");
		BooleanMatrix mat = new BooleanMatrix();
		setMatrix(mat);
		setPaintArray();
		setPointArray();
		isCon = true;
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		xp = (maxWide+1)*elementBlock;
		yp = (maxLevel+2)*elementBlock;
		setSize(xp, yp);
		
		Container c = getContentPane();
		setBackground( bkColor );
		c.setBackground(getBackground());
		requestFocus();
		setVisible(true);
	}
	
/**
* 隣接マトリクスを設定し、描画を行います。
* 非循環で、双方向の枝がないグラフ以外は動作保障されません。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public GraphPaint(BooleanMatrix adjacencyMatrix){
		super("LFG");
		setMatrix(adjacencyMatrix);
		setPaintArray();
		setPointArray();
		isCon = true;
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		xp = (maxWide+1)*elementBlock;
		yp = (maxLevel+2)*elementBlock;
		setSize(xp, yp);
		
		Container c = getContentPane();
		setBackground( bkColor );
		c.setBackground(getBackground());
		requestFocus();
		setVisible(true);
	}

/**
* 隣接マトリクスとレベル情報、座標位置を設定し、描画を行います。
* 主にLFGや課題系列グラフといったを描画するための非循環で、双方向の枝がなく、
* 縦軸にレベル以外の値をとらないグラフを描画するためのコンストラクタです。
* @param adjacencyMatrix 隣接マトリクス
* @param pointArray 要素の配置座標を格納した配列
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
* @see jp.ac.takushoku_u.cs.ArrangePoint
* @see jp.ac.takushoku_u.cs.ArrangePoint#getPointArray()
*/
	public GraphPaint(BooleanMatrix adjacencyMatrix, int[][] pointArray){
		super("LFG");
		setMatrix(adjacencyMatrix);
		setPointArray(pointArray);
		isCon = true;
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		xp = getMaxXPoint(pointArray) + elementBlock;
		yp = getMaxYPoint(pointArray) + elementBlock;
		setSize(xp, yp);
		
		Container c = getContentPane();
		setBackground( bkColor );
		c.setBackground(getBackground());
		requestFocus();
		setVisible(true);
	}
	
/**
* 隣接マトリクスとレベル情報、座標位置を設定し、描画を行います。
* 縦軸の設定が必要であるグラフ、主にSSグラフを描くためのコンストラクタです。
* このコンストラクタを用いると縦軸が描画されます。
* @param adjacencyMatrix 隣接マトリクス
* @param pointArray 要素の配置座標を格納した配列
* @param max 縦軸の最大値
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
* @see jp.ac.takushoku_u.cs.SSGraphPoint
* @see jp.ac.takushoku_u.cs.SSGraphPoint#getPointArray()
*/
	public GraphPaint(BooleanMatrix adjacencyMatrix, int[][] pointArray, double max){
		super("SSGraph");
		setMatrix(adjacencyMatrix);
		setPointArray(pointArray);
		isCon = true;
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		xp = getMaxXPoint(pointArray) + elementBlock;
		yp = 700;//((getMaxYPoint(pointArray) / elementBlock)+2) * elementBlock;
		maxValue = max;
		setSize(xp, yp);
		
		Container c = getContentPane();
		setBackground( bkColor );
		c.setBackground(getBackground());
		requestFocus();
		setVisible(true);
	}
	
/**
* グラフを描画します。
*/
	public void paint(Graphics g){
		
		//縦軸を描画します
		if(!Double.isNaN(maxValue)){
			
			//描画色の設定
			g.setColor(rowColor);
			
			//縦軸線の描画
			g.drawLine(elementBlock*4/5, elementBlock+elmentSize/2, elementBlock*4/5, (yp - elementBlock)+elmentSize/2);
			
			//縦軸の間隔
			int interval;
			if((yp * 1.0 - elementBlock) % (scale+1.0) >= (scale+1.0) / 2.0 ){
				interval = (yp - elementBlock) / (scale+1);
			}
			else{
				interval = (yp - elementBlock) / (scale+1) + 1;
			}
			for(int i = 0; i <= scale; i++){
				g.setColor(rowColor);
				g.drawLine(elementBlock*3/5,interval * i + (elementBlock + elmentSize/2),
					elementBlock*4/5,interval * i + (elementBlock + elmentSize/2));
				
				//数値
				g.setColor(rowNumberColor);
				g.drawString(""+(maxValue - (maxValue*i) / scale ), elementBlock*1 / 10,
					interval * i + (elementBlock + elmentSize/2));
			}
			System.out.println();
		}
		
		
		//各要素を描画します
		for(int i = 0; i < matrix.getRowLength(); i++){
			g.setColor(elColor);
			g.fillOval(paintPoint[i][0], paintPoint[i][1], elmentSize, elmentSize); //(x,y,大きさ,大きさ)
			g.setColor(elNumberColor);
			if(i < 10){
				g.drawString(i+"",paintPoint[i][0]+(elmentSize/4),paintPoint[i][1]+(elmentSize*3/4));
			}
			else{
				g.drawString(i+"",paintPoint[i][0]+(elmentSize/10),paintPoint[i][1]+(elmentSize*3/4));
			}
		}
		
		
		//各枝を描画します
		for(int i = 0; i < matrix.getRowLength(); i++){
			g.setColor(arColor);
			for(int j = 0; j < matrix.getColLength(); j++){
				if(matrix.getCell(i,j)){
					
					//矢印の描画
					//x0 始点のx座標,始点のy座標
					//x1 終点のx座標,y1 終点のy座標,arrow 矢の長さ
					int x0, y0, x1, y1;
					
					//上から下へ引くとき
					if(paintPoint[j][1] > paintPoint[i][1]){
						x0 = paintPoint[i][0] + elmentSize / 2;
						y0 = paintPoint[i][1] + elmentSize;
						x1 = paintPoint[j][0] + elmentSize / 2;
						y1 = paintPoint[j][1];
					}
					//同じレベルに引くとき
					else if(paintPoint[j][1] == paintPoint[i][1]){
						if(paintPoint[j][0] >= paintPoint[i][0]){
							x0 = paintPoint[i][0] + elmentSize;
							y0 = paintPoint[i][1] + elmentSize / 2;
							x1 = paintPoint[j][0];
							y1 = paintPoint[j][1] + elmentSize / 2;
						}
						else{
							x0 = paintPoint[i][0];
							y0 = paintPoint[i][1] + elmentSize / 2;
							x1 = paintPoint[j][0] + elmentSize;
							y1 = paintPoint[j][1] + elmentSize / 2;
						}
					}
					//それ以外
					else{
						x0 = paintPoint[i][0] + elmentSize / 2;
						y0 = paintPoint[i][1];
						x1 = paintPoint[j][0] + elmentSize / 2;
						y1 = paintPoint[j][1] + elmentSize;
					}
					
					int arrow = 7;
					double theta;
					
					//矢を描く時のx,y
					int x,y;
					double dt = Math.PI / 6.0;
					theta = Math.atan2((double)(y1-y0),(double)(x1-x0));
					g.drawLine(x0,y0,x1,y1);
					x = x1-(int)(arrow*Math.cos(theta-dt));
					y = y1-(int)(arrow*Math.sin(theta-dt));
					g.drawLine(x1,y1,x,y);
					x = x1-(int)(arrow*Math.cos(theta+dt));
					y = y1-(int)(arrow*Math.sin(theta+dt));
					g.drawLine(x1,y1,x,y);
				}
			}
		}
	}
	
	
/**
* 最大レベルを返します。
* @param levelArray レベルが入った配列
* @return 最大レベル
*/
	private int getRowMaxLevel(int[] levelArray){
		int num = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] > num){
				num = levelArray[i];
			}
		}
		return num;
	}
	
/**
* 最大横幅を返します。
* @param levelArray レベルが入った配列
* @return 最大横幅
*/
	private int getColMaxLevel(int[] levelArray){
		int maxCount = 0;
		for(int i = 0; i < levelArray.length; i++){
			int count = 0;
			for(int j = 0; j < levelArray.length; j++){
				if(levelArray[j] == i){
					count++;
				}
			}
			if(count > maxCount){
				maxCount = count;
			}
		}
		
		return maxCount;
	}
	
/**
* 指定したレベルの横幅を探索します。
* @param level 指定するレベル
* @param levelArray レベルが入った配列
* @return 横幅
*/
	private int getColLevel(int level, int[] levelArray){
		int count = 0;
		for(int i = 0; i < levelArray.length; i++){
			if(levelArray[i] == level){
				count++;
			}
		}
		
		return count;
	}
	
/**
* 描画を行うグラフを隣接マトリクスとして設定します。
* @param adjacencyMatrix 隣接マトリクス
* @exception NotSquareMatrixException 設定したマトリクスが正方行列ではない場合
*/
	public void setMatrix(BooleanMatrix adjacencyMatrix){
		matrix = new BooleanMatrix();
		reachMatrix = new BooleanMatrix();
		if(adjacencyMatrix.isSquare()){
			matrix.matrixCopy(adjacencyMatrix);
			reachMatrix.matrixCopy(matrix.getReachMatrix());
			if(isCon){
				setPaintArray();
			}
		}
		else{
			throw new NotSquareMatrixException("row("+ adjacencyMatrix.getRowLength() +
				"),col("+adjacencyMatrix.getColLength()+")");
		}
	}
	
/**
* 描画情報のための配列を設定します。
* 情報は隣接マトリクスから自動で生成されます。
*/
	private void setPaintArray(){
		int[] returnArray = new int[matrix.getRowLength()];
		System.arraycopy(getLevelArray(), 0, returnArray, 0, matrix.getRowLength());
		setPaintArray(returnArray);
	}
	
/**
* グラフを描く際のレベルを配列として返します。
* @return レベルを格納した配列
*/
	private int[] getLevelArray(){
		int[] returnArray = new int[matrix.getRowLength()];
		BooleanMatrix reachMatrix= new BooleanMatrix();
		reachMatrix.matrixCopy(matrix.getReachMatrix());
		
		//配列の初期化
		for(int i = 0; i < returnArray.length; i++){
			returnArray[i] = -1;
		}
		
		//ルートを決める
		for(int i = 0; i < returnArray.length; i++){
			if(matrix.getRowSum(i) == 0){
				returnArray[i] = 0;
			}
		}
		
		boolean flag = true;
		
		//レベルを格納する
		for(int level = 0; level < returnArray.length && flag; level++){
			for(int i = 0; i < returnArray.length; i++){
				
				//レベルが決定していなければ
				if(returnArray[i] == -1){
					int count = 1;
					for(int j = 0; j < returnArray.length; j++){
						
						//枝の先の要素はレベルが決定しているか
						if(reachMatrix.getCell(i, j) && returnArray[j] >= 0 &&
							returnArray[j] < level){
							count++;
						}
						else if(matrix.getCell(i, j) && matrix.getCell(j, i)){
							count++;
						}
					}
					if(count == reachMatrix.getRowSum(i)){
						returnArray[i] = level;
					}
				}
			}
		}
		
		//ルートの位置を変更する
		for(int i = 0; i < returnArray.length; i++){
			if(returnArray[i] == 0){
				int count = 0;
				for(int j = 0; j < matrix.getRowLength(); j++){
					if(matrix.getCell(j,i)){
						if(count < returnArray[j]){
							count = returnArray[j];
						}
					}
				}
				returnArray[i] = count - 1;
			}
		}
		
		return returnArray;
	}
	
/**
* 描画情報のための配列を設定します。
* @param levelArray 各要素をどのレベルに配置するかレベル番号を格納した配列
* @exception DisagreementArraySizeException マトリクスと配列のサイズが一致しない場合
*/
	public void setPaintArray(int[] levelArray){
		if(matrix.getRowLength() == levelArray.length){
			maxLevel = getRowMaxLevel(levelArray);
			maxWide = getColMaxLevel(levelArray);
			elementNumber = levelArray.length;
			
			paintLevel = new int[levelArray.length];
			System.arraycopy(levelArray, 0, paintLevel, 0, levelArray.length);
			paintWide = new int[maxLevel+1];
			for(int i = 0; i <= maxLevel; i++){
				paintWide[i] = getColLevel(i, levelArray);
			}
			setPointArray();
		}
		else{
			throw new DisagreementArraySizeException("matrix("+matrix.getRowLength()+
				"),array("+levelArray.length+")");
		}
	}
	
/**
* 描画位置指定のための配列を設定します。
* 情報は隣接マトリクスから自動で生成されます。
*/
	private void setPointArray(){
		paintPoint = new int[elementNumber][2];
		setPaint();
	}
	
/**
* 描画位置指定のための配列を設定します。
* @param pointArray 位置情報配列
*/
	public void setPointArray(int[][] pointArray){
		if(matrix.getRowLength() == pointArray.length){
			paintPoint = new int[pointArray.length][2];
			for(int i = 0; i < pointArray.length; i++){
				System.arraycopy(pointArray[i], 0, paintPoint[i], 0, pointArray[i].length); 
			}
			
		}
		else{
			throw new DisagreementArraySizeException("matrix("+matrix.getRowLength()+
				"),array("+pointArray.length+")");
		}
	}
	
/**
* 描画位置情報を設定します。
*/
	private void setPaint(){
		//各レベルで既に描いた要素数をカウントします。
		int[] countArray = new int [maxLevel+1];
		for(int i = 0; i <= maxLevel; i++){
			if(maxWide%2==0){
				countArray[i] = (maxWide + 1 - paintWide[i]) / 2;
			}
			else{
				countArray[i] = (maxWide - paintWide[i]) / 2;
			}
		}
		
		
		//各要素のx座標とy座標を決めます
		for(int i = 0; i < elementNumber; i++){
				
			//横が偶数個
			if(paintWide[paintLevel[i]]%2 == 0){
				if(countArray[paintLevel[i]]+1 == maxWide/2+1){
					countArray[paintLevel[i]]++;
				}
				paintPoint[i][0] = (countArray[paintLevel[i]] + 1) * elementBlock;
				countArray[paintLevel[i]]++;
			}
			
			//横が奇数個
			else{
				paintPoint[i][0] = (countArray[paintLevel[i]] + 1) * elementBlock;
				countArray[paintLevel[i]]++;
			}
			
			//y座標の高さ
			paintPoint[i][1] = (paintLevel[i]+1) * elementBlock;
		}
	}
	
/**
* 描画情報からxの最大値を求めます。
* @param pointArray 位置情報配列
*/
	private int getMaxXPoint(int[][] pointArray){
		int max = -1;
		for(int i = 0; i < pointArray.length; i++){
			if(pointArray[i][0] > max){
				max = pointArray[i][0];
			}
		}
		return max;
	}
	
/**
* 描画情報からyの最大値を求めます。
* @param pointArray 位置情報配列
*/
	private int getMaxYPoint(int[][] pointArray){
		int max = -1;
		for(int i = 0; i < pointArray.length; i++){
			if(pointArray[i][1] > max){
				max = pointArray[i][1];
			}
		}
		return max;
	}
	
/**
* 描画される有向枝の色を変更します。
* 設定しない場合はデフォルト色(red)となります。
* @param c 設定する色
*/
	public void setArrowColor(Color c){
		arColor = c;
		repaint();
	}
	
/**
* 縦軸線の色を変更します。
* 設定しない場合はデフォルト色(magenta)となります。
* @param c 設定する色
*/
	public void setRowLineColor(Color c){
		rowColor = c;
		repaint();
	}
	
/**
* 縦軸の目盛(数値)の色を変更します。
* 設定しない場合はデフォルト色(darkGray)となります。
* @param c 設定する色
*/
	public void setRowLineNumberColor(Color c){
		rowNumberColor = c;
		repaint();
	}
	
/**
* 描画する要素の色を変更します。
* 設定しない場合はデフォルト色(cyan)となります。
* @param c 設定する色
*/
	public void setElementColor(Color c){
		elColor = c;
		repaint();
	}
	
/**
* 要素内の番号の色を変更します。
* 設定しない場合はデフォルト色(black)となります。
* @param c 設定する色
*/
	public void setElementNumberColor(Color c){
		elNumberColor = c;
		repaint();
	}
	
}