package jp.ac.takushoku_u.cs;

/**
* OrderPairクラスはグラフの要素の順序対を保持するクラスです。<BR>
* @version 1.0.1
* @author 河原　慎治(Kawahara Shinji)
* @see OrderPairSet
* @see MathEdge
*/
/*
* 最終更新日 2005年11月28日
*/
public class OrderPair{
	
/**
* 枝の始点の要素番号
*/
	private int start;
	
/**
* 枝の終点の要素番号
*/
	private int goal;
	
/**
* 初期要素番号
*/
	private static final int INITIALIZE_BEGIN_ELEMENT = 0;
	
/**
* 初期要素番号
*/
	private static final int INITIALIZE_END_ELEMENT = 1;
	
/**
* デフォルトの要素で要素対集合を設定します。
* 始点0、終点1で設定されます。
*/
	public OrderPair(){
		setBegin(INITIALIZE_BEGIN_ELEMENT);
		setEnd(INITIALIZE_END_ELEMENT);
	}
	
/**
* 枝を設定します。
* @param begin 順序対の始点
* @param end 順序対の終点
*/
	public OrderPair(int begin, int end){
		setBegin(begin);
		setEnd(end);
	}
	
/**
* 順序対の始点の要素を返します。
* @return 順序対の始点要素
*/
	public int getBegin(){
		return start;
	}
/**
* 順序対の終点の要素を返します。
* @return 順序対の終点要素
*/
	public int getEnd(){
		return goal;
	}
	
/**
* 指定されたOrderPairは同じ順序対を表しているかを比較します。
* @param od 比較するOrderPair
* @return 同じ順序対を表していれば true
*/
	public boolean isSameEdge(OrderPair od){
		return (getBegin() == od.getBegin()) && (getEnd() == od.getEnd());
	}
	
/**
* 順序対の始点の設定をします。
* @param begin 始点の要素
* @exception NegativeValueException 指定された要素番号が負である場合
*/
	private void setBegin(int begin){
		if(begin >= 0){
			start = begin;
		}
		else{
			throw new NegativeValueException("begin("+begin+")");
		}
	}
/**
* 順序対の終点の設定をします。
* @param end 終点の要素
* @exception NegativeValueException 指定された要素番号が負である場合
*/
	private void setEnd(int end){
		if(end >= 0){
			goal = end;
		}
		else{
			throw new NegativeValueException("end("+ end +")");
		}
	}
	
/**
* 指定された値は負ではないかチェックします。
* @param value チェックする値
* @return 値が正または0であれば true
*/
	private boolean isPositive(int value){
		return value >= 0;
	}
}